"""
This file is part of AppReport.

AppReport is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, version 3 of the License.

AppReport is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with AppReport.  If not, see <http://www.gnu.org/licenses/>.
"""

import base64
import app_report


class Decoder(object):

    @staticmethod
    def decode_base64(encoded):
        return base64.b64decode(encoded)

    @classmethod
    def decode(cls, encoded, encoding='base64'):
        supported = ('base64', )

        if not encoded:
            raise app_report.errors.DecoderError("Encoded can't be blank!")

        elif not encoding:
            raise app_report.errors.DecoderError("Encoding can't be blank!")

        elif encoding not in supported:
            raise app_report.errors.DecoderError("Encoding '%s' not supported, only %s." % (encoding, supported))

        return getattr(cls, "decode_%s" % encoding)(encoded)



"""
This file is part of AppReport.

AppReport is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, version 3 of the License.

AppReport is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with AppReport.  If not, see <http://www.gnu.org/licenses/>.
"""


class AppReportError(Exception):
    pass


class ValidationError(AppReportError):
    pass


class DecoderError(AppReportError):
    pass


# configuration


class ConfigurationError(AppReportError):
    pass


class RequiredConfigurationError(ConfigurationError):
    pass


class InvalidConfigurationError(ConfigurationError):
    pass


class InvalidConfigurationKeyError(ConfigurationError):
    pass


# API


class APIError(AppReportError):
    pass


class APIResponseError(APIError):
    pass



"""
This file is part of AppReport.

AppReport is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, version 3 of the License.

AppReport is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with AppReport.  If not, see <http://www.gnu.org/licenses/>.
"""

from time import time
import app_report


class Base(object):

    def __init__(self):
        app_report.AppReport.settings.validates()
        self.params = {}

    def params_to_sign(self):
        return []

    def sign_params(self, params, ordered_keys):
        string = ''.join(str(params[key]) for key in ordered_keys)
        return app_report.Signer.sign(secret_key=app_report.AppReport.settings.secret_key, string=string)

    def signed_params(self):
        params = {}

        # include params that must be signed
        for param in self.params_to_sign():
            params[param] = self.params.get(param, '')

        # include configured keys
        params.update({
            'app_name': app_report.AppReport.settings.app_name,
            'expires': int(time()) + app_report.AppReport.settings.expires_signed_url_after,
        })

        # sign params
        params.update({
            'signature': self.sign_params(params, self.params_to_sign()),
            'access_key': app_report.AppReport.settings.access_key
        })

        # include not signed params
        _params = self.params.copy()
        _params.update(params)

        return _params



"""
This file is part of AppReport.

AppReport is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, version 3 of the License.

AppReport is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with AppReport.  If not, see <http://www.gnu.org/licenses/>.
"""

from urlparse import urljoin
import simplejson as json
import requests
import app_report


class Client(object):
    endpoint = 'http://reports.simpleservic.es'
    connection = requests

    @classmethod
    def build_url(cls, path):
        return urljoin(cls.endpoint, path)

    @classmethod
    def post(cls, path, params={}):
        url = cls.build_url(path)
        params = json.dumps(params)
        headers = {'content-type': 'application/json'}

        response = cls.connection.post(url, data=params, headers=headers)
        cls.raise_error_messages(response.json())

        return response

    @classmethod
    def raise_error_messages(cls, json_data):
        if 'messages' in json_data and json_data['messages'].get('has_any_error', False):
            messages = ' '.join({message.get('message', 's') for message in json_data.get('messages', []).get('messages', [])})
            raise app_report.errors.APIError('[%s]' % messages)



"""
This file is part of AppReport.

AppReport is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, version 3 of the License.

AppReport is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with AppReport.  If not, see <http://www.gnu.org/licenses/>.
"""

import app_report.errors


class Jasper(object):

    attribute_keys = (
        'template_name',
        'data_type',
        'data',
        'xpath_expression',
        'args'
    )

    def __init__(self, **attributes):
        for key in self.attribute_keys:
            setattr(self, key, attributes.get(key, None))

    # validation

    def validates_all_attributes(self):
        for key in self.attribute_keys:
            self.validates_presence_of(key)

    # alias
    validates = validates_all_attributes

    def validates_presence_of(self, attribute_name):
        if not getattr(self, attribute_name):
            raise app_report.errors.ValidationError("%s is required" % attribute_name)

    def validates_template_name(self):
        self.validates_presence_of('template_name')

    def validates_data_type(self):
        pass

    def validates_data(self):
        pass

    def validates_xpath_expression(self):
        pass

    def validates_args(self):
        pass



"""
This file is part of AppReport.

AppReport is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, version 3 of the License.

AppReport is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with AppReport.  If not, see <http://www.gnu.org/licenses/>.
"""

import hmac
from hashlib import sha1
import base64


class Signer(object):

    @staticmethod
    def to_utf8(string):
        if isinstance(string, unicode):
            return string.encode('utf-8')

        return unicode(string, 'utf-8').encode('utf-8')

    @classmethod
    def sign(cls, secret_key, string):
        string = cls.to_utf8(string)
        signed = hmac.new(secret_key, string, sha1).digest()

        return base64.b64encode(signed)



"""
This file is part of AppReport.

AppReport is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, version 3 of the License.

AppReport is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with AppReport.  If not, see <http://www.gnu.org/licenses/>.
"""

import errors
from configuration import Configuration
from client import Client
import report
import api
from signer import Signer
from decoder import Decoder


class AppReport:
    settings = Configuration()

    @classmethod
    def configure(cls, config):
        for key, value in config.items():

            if key not in cls.settings.keys:
                raise errors.InvalidConfigurationKeyError("invalid configuration key: %s" % key)

            setattr(cls.settings, key, value)

        cls.settings.validates()
        return cls.settings



"""
This file is part of AppReport.

AppReport is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, version 3 of the License.

AppReport is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with AppReport.  If not, see <http://www.gnu.org/licenses/>.
"""

import app_report.errors


class Configuration(object):

    keys = [
        'app_name',
        'access_key',
        'secret_key',
        'expires_signed_url_after',
    ]

    def __init__(self):
        # expire urls after 10 minutes, by default.
        self._expires_signed_url_after = 600

        for key in self.keys:
            if not hasattr(self, key):
                setattr(self, key, None)

    @property
    def expires_signed_url_after(self):
        return self._expires_signed_url_after

    @expires_signed_url_after.setter
    def expires_signed_url_after(self, value):
        if not isinstance(value, int) or value < 1:
            raise app_report.errors.InvalidConfigurationError("value must be an integer > 0, not %s" % value)

        self._expires_signed_url_after = value

    def validates(self):
        for key in self.keys:
            if not getattr(self, key):
                raise app_report.errors.RequiredConfigurationError("%s is required" % key)



from setuptools import setup, find_packages

__version__ = '0.0.1'


def long_description():
    with open('README.md') as f:
        return f.read()


def requirements():
    with open('requirements.txt') as reqs:
        return [line for line in reqs.read().split('\n') if (line and not line.startswith('#'))]

setup(
    name='app_report',
    packages=find_packages(),
    version=__version__,
    description='Python client to AppReport API',
    long_description=long_description(),
    author="Lucas D'Avila",
    author_email='lucas@lucasdavi.la',
    url='https://github.com/simple_services/app_report-python',
    keywords=['pdf', 'report', 'jasper', 'api'],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Environment :: Web Environment',
        'Development Status :: 3 - Alpha'
    ],
    install_requires=requirements()
)



"""
This file is part of AppReport.

AppReport is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, version 3 of the License.

AppReport is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with AppReport.  If not, see <http://www.gnu.org/licenses/>.
"""

import app_report
from app_report.api import Base
import simplejson as json


class Jasper(Base):

    @staticmethod
    def params_to_sign():
        return ('app_name', 'template_name', 'expires')

    @staticmethod
    def validates_report(report):

        if not isinstance(report, app_report.report.Jasper):
            error_msg = "report must be a instance of app_report.report.Jasper not %s" % type(report)
            raise app_report.errors.ValidationError(error_msg)

    def build(self, report):
        self.report = report
        self.validates_report(self.report)
        self.report.validates()

        for key in self.report.attribute_keys:
            self.params[key] = getattr(self.report, key)

        path = '/api/v1/factory/jasper/build.json'
        response = app_report.Client.post(path, self.signed_params())

        return self.decode_report(response.json())

    @staticmethod
    def decode_report(response_body):
        report = response_body.get('report', '')

        if not report:
            raise app_report.errors.APIResponseError("API returned a blank report!")

        return app_report.Decoder.decode(report.get('encoded', ''), report.get('encoding', ''))
